package ocular;

import java.util.*;
import java.util.Map.Entry;

public class CallDB {
	public class CallInfo{
		public long refId=0;
		public String s="";
		public String to="";
		public String from="";
		public String to2="";
		public String from2="";
		public String startTime="";
		public String updateTime="";
		public int fromTraced=-1;
		public int toTraced=-1;
		public CallInfo(String s,String to,String from,String startTime){
			this.s=s;
			this.to=(to!=null)?to:"";
			this.from=(from!=null)?from:"";
			this.startTime=(startTime!=null)?startTime:"";
		}
	}
	private int maxSNum=50;
	private Map<Long,CallInfo> calls;
	private HashMap<Long,Long> refCalls;
	private List<Long> table;
	private int changed=0;
	private boolean skipNew=false;
	private String trace0=null;
	private String trace1=null;
	
	private Map<String,Integer> stats;
	
	public CallDB(int maxSNum){
		this.maxSNum=maxSNum;
		calls=Collections.synchronizedMap(new HashMap<Long,CallInfo>());
		table=Collections.synchronizedList(new ArrayList<Long>(maxSNum));
		refCalls=new HashMap<Long,Long>();
		stats=Collections.synchronizedMap(new HashMap<String,Integer>());
	}
	public boolean update(long id,long rid,String newS,String to,String from){
		if(id==0) return false;
		synchronized (calls) {
			CallInfo ci;
			if(rid!=0){
				ci=calls.get(rid);
				if(null!=ci){
					ci.refId=id;
					refCalls.put(id, rid);
				}else{
					ci=calls.get(id);
				}
			}else{
				Long cr=refCalls.get(id);
				if(cr!=null) ci=calls.get(cr);
				else ci=calls.get(id);
			}
			if(ci!=null){
				ci.s+=" "+newS;
				ci.updateTime=getCurrentTime();
				if(to!=null){
					if(trace0!=null&&ci.toTraced==-1) ci.toTraced=to.indexOf(trace0);
					if(trace1!=null&&ci.toTraced==-1) ci.toTraced=to.indexOf(trace1);

					if(!ci.to.equals(""))ci.to2=to;
					else ci.to=to;
				}
				if(from!=null){
					if(trace0!=null&&ci.fromTraced==-1) ci.fromTraced=from.indexOf(trace0);
					if(trace1!=null&&ci.fromTraced==-1) ci.fromTraced=from.indexOf(trace1);
					if(!ci.from.equals(""))ci.from2=from;
					else ci.from=from;
				}
				
			} else if(!skipNew) {
				calls.put(id,new CallInfo(newS,to,from,getCurrentTime()));
				if(table.size()==maxSNum){
					refCalls.remove(calls.get(table.get(0)).refId);
					calls.remove(table.get(0));
					table.remove(0);
				}
				table.add(id);
			} else return false;
		}
		return true;
		
	}
	private String getCurrentTime() {
		Calendar c=Calendar.getInstance();
		return c.get(Calendar.HOUR)+":"+c.get(Calendar.MINUTE)+":"+c.get(Calendar.SECOND)+"."+c.get(Calendar.MILLISECOND);
	}
	public int getNum() {
		synchronized (calls) {
		return calls.size();
		}
	}
	public String getScrolled(int idx) {
		synchronized (calls) {
		return calls.get(table.get(idx)).s;
		}
	}
	public int getChanged() {
		return changed;
	}
	public boolean skipNew() {
		return (skipNew=true);
	}
	public boolean processNew() {
		return (skipNew=false);
	}
	public String getFromIn(int idx) {
		synchronized (calls) {
			return calls.get(table.get(idx)).from;
			}
	}
	public String getFromOut(int idx) {
		synchronized (calls) {
			return calls.get(table.get(idx)).from2;
			}
	}
	public String getToIn(int idx) {
		synchronized (calls) {
			return calls.get(table.get(idx)).to;
			}
	}
	public String getToOut(int idx) {
		synchronized (calls) {
			return calls.get(table.get(idx)).to2;
			}
	}
	public String getStartTime(int idx) {
		synchronized (calls) {
			return calls.get(table.get(idx)).startTime;
			}
	}
	public String getUpdateTime(int idx) {
		synchronized (calls) {
			return calls.get(table.get(idx)).updateTime;
			}
	}
	public String getStatusString() {
			return ""+stats.size()+" "+calls.size()+"/"+refCalls.size()+"/"+table.size();
	}
	public void trace(String num0, String num1) {
		trace0=(num0!=null&&num0.length()!=0)?num0:null;
		trace1=(num1!=null&&num1.length()!=0)?num1:null;
		doRetroTrace();
	}
	private void doRetroTrace() {
		synchronized (calls) {
			for (CallInfo ci:calls.values()) {
				if(trace0!=null) doTrace0(ci,trace0);
				else if(trace1!=null) doTrace0(ci,trace1);
				else ci.toTraced=ci.fromTraced=-1;
				if(trace1!=null) doTrace1(ci,trace1);
				else if(trace0!=null) doTrace0(ci,trace0);
			}
		}
		
	}
	private void doTrace0(CallInfo ci, String trace) {
		ci.toTraced=doTrace(ci.to,trace);
		ci.fromTraced=doTrace(ci.from,trace);
		if(ci.toTraced==-1) ci.toTraced=doTrace(ci.to2,trace);
		if(ci.fromTraced==-1) ci.fromTraced=doTrace(ci.from2,trace);
	}
	private void doTrace1(CallInfo ci, String trace) {
		if(ci.toTraced==-1) ci.toTraced=doTrace(ci.to,trace);
		if(ci.fromTraced==-1) ci.fromTraced=doTrace(ci.from,trace);
		if(ci.toTraced==-1) ci.toTraced=doTrace(ci.to2,trace);
		if(ci.fromTraced==-1) ci.fromTraced=doTrace(ci.from2,trace);
	}
	private int doTrace(String num, String trace) {
		if(num!=null&&num.length()>0) return num.indexOf(trace);
		return -1;
	}
	public boolean isTraced(int idx) {
		synchronized (calls) {
			CallInfo ci=calls.get(table.get(idx));
			return (ci.toTraced!=-1||ci.fromTraced!=-1);
			}
	}
	/**
	 * @param idx
	 * @return 0 - none, 1 - to, 2 - from, 3 - both
	 */
	public int getTraced(int idx) {
		synchronized (calls) {
			CallInfo ci=calls.get(table.get(idx));
			if(ci.toTraced!=-1&&ci.fromTraced==-1) return 1; //to
			if(ci.toTraced==-1&&ci.fromTraced!=-1) return 2; //from
			if(ci.toTraced!=-1&&ci.fromTraced!=-1) return 3; //both
			return 0;
			}
	}
	public void inc(String statStr) {
		synchronized (stats) {
			if(stats.containsKey(statStr)) stats.put(statStr,stats.get(statStr).intValue()+1);
			else stats.put(statStr,1);
		}
	
	}
	public Set<Map.Entry<String, Integer>> getStats() {
		return stats.entrySet();
	}
}
